#ifndef VIEWMANAGER_H
#define VIEWMANAGER_H

/*                                                                      
    This file is part of KAddressBook.                                  
    Copyright (c) 2002 Mike Pilone <mpilone@slac.com>                   
                                                                        
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or   
    (at your option) any later version.                                 
                                                                        
    This program is distributed in the hope that it will be useful,     
    but WITHOUT ANY WARRANTY; without even the implied warranty of      
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the        
    GNU General Public License for more details.                        
                                                                        
    You should have received a copy of the GNU General Public License   
    along with this program; if not, write to the Free Software         
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           
                                                                        
    As a special exception, permission is given to link this program    
    with any edition of Qt, and distribute the resulting executable,    
    without including the source code for Qt in the source distribution.
*/                                                                      

#include <qwidget.h>
#include <qdict.h>
#include <qstringlist.h>

#include <kabc/field.h>

class QComboBox;
class QLineEdit;
class QWidgetStack;
class QResizeEvent;
class KConfig;
class ViewWrapper;
class KAddressBookView;
class ContactEntryList;
class JumpButtonBar;

/** The view manager manages the views and everything related to them. The
* manager will load the views at startup and display a view when told to
* make one active.
*
* The view manager will also create and manage all dialogs directly related to
* views (ie: AddView, ConfigureView, DeleteView, etc).
*/
class ViewManager : public QWidget
{
  Q_OBJECT
  
  public:
    ViewManager(ContactEntryList *doc, KConfig *config, 
                QWidget *parent = 0, const char *name = 0);
    ~ViewManager();
    
    /** Sets the given view active. This usually means raising the
    * view to the top of the widget stack and telling it
    * to refresh.
    */
    void setActiveView(const QString &name);
    
   /** Destroys all the currently loaded views. It is important that
    * after calling this method you call setActiveView before
    * the user has a chance to interact with the gui, since no
    * views will be loaded.
    */
    void unloadViews();
    
    /** Returns a list of all the defined views. This list is guaranteed
    * to always contain at least one view. This list is the 'defined' views,
    * not necessarily the loaded views. However the view will be loaded if
    * it becomes active.
    */
    const QStringList &viewNames() { return mViewNameList; }
    
    /* Return a list of all the uids of selected contacts.
    */
    QStringList selectedUids();
    
    /** Used to enable or disable the incremental search widget.
    *
    * @param visible True for the widget to be visible, false otherwise
    */
    void setIncrementalSearchVisible(bool visible);
    
    /** Used to enable or disable the jump button bar.
    *
    * @param visible True for the widget to be visible, false otherwise
    */
    void setJumpButtonBarVisible(bool visible);
    
  public slots:
    /** Reads the config file.
    */
    void readConfig();
    
    /** Writes the config file.
    */
    void writeConfig();
    
    /** Sends an email to all the selected addressees. This is done by
    * asking the view for a string of "To:'s" and then asking
    * KDE to open the mailer with the information.
    */
    virtual void sendMail();
    
    /** This slot will delete all the selected entries. This method should
    * be called just 'delete' to be consistant with the other edit methods,
    * but good 'ol C++ wouldn't like that -mpilone
    */
    void deleteAddressee();
    
    /** Paste a contact into the addressbook from the clipboard.
    */
    void paste();
    
    /** Copy a contact from the view into the clipboard. This method will
    * copy all selected contacts into the clipboard at once.
    */
    void copy();
    
    /** Cut a contact from the view into the clipboard. This method will
    * cut all selected contacts into the clpboard at once.
    */
    void cut();
    
    /** Selects the given addressee or all addressees if uid == QString::null
    */
    void setSelected(QString uid = QString::null, bool selected = true); 
    
   /** Refreshes the active view. 
   *
   * @param uid Only refresh the selected uid. If it is QStrign::null, the
   * entire view will be refreshed.
    */
    void refresh(QString uid = QString::null);
    
    /** Launches the ConfigureView dialog for the active view.
    */
    void modifyView();
    
    /** Deletes the current view and makes another view active.
    */
    void deleteView();
    
    /** Displays the add view dialog. If the user confirms, the view
    * will be added.
    */
    void addView();
    
  protected slots:
   /** Called whenever the text in the incremental search line edit
    * is changed.
    */
    void incrementalSearchTextChanged(const QString &text);
    
    /** Called whenever return is pressed in the incremental search
    * line edit.
    */
    void incrementalSearchReturnPressed();
    
    /** Called whenever the incremental search combo box is activated (when
    * an item is selected).
    */
    void incrementalSearchComboActivated(const QString &text);
    
    /** Called whenever the user selects a button from the jump bar
    */
    void jumpToLetter(const QChar &ch);
    
  signals:
    /** Called whenever the user selects an entry in the view.
    */
    void selected(const QString &uid);
    
    /** called whenever the user activates an entry in the view.
    */
    void executed(const QString &uid);
    
    /** Emitted whenever the address book is modified in some way.
    */
    void modified();
    
    /** Emitted whenever the view configuration changes. This can happen
    * if a user adds a new view or removes a view.
    *
    * @param newActive This is the view that should be made active. If this
    * is QString::null, than the current active can remain that way.
    */
    void viewConfigChanged(const QString &newActive);
    
  private:
    /** Create a view wrapper for each type of view we know about
    */
    void createViewWrappers();
    
    /** Creates the GUI components
    */
    void initGUI();
    
   /** Populates the incremental search combo box.
    */
    void refreshIncrementalSearchCombo();
    
    QStringList mViewNameList;
    QDict<ViewWrapper> mViewWrapperDict;
    QDict<KAddressBookView> mViewDict;
    KAddressBookView *mActiveView;
    ContactEntryList *mDocument;
    KConfig *mConfig;
    QWidgetStack *mViewWidgetStack;
    QLineEdit *mIncrementalSearchEdit;
    QComboBox *mIncrementalSearchCombo;
    KABC::Field::List mIncrementalSearchFields;
    QWidget *mIncrementalSearchWidget;
    JumpButtonBar *mJumpButtonBar;
    
};

#endif
